# Bayesian Method (2nd edition)
# written by Jeff Gill
# http://jgill.wustl.edu/BMSBSA2/

# R scripts in ch. 9
# typed by Jeyong Sohn (except for scripts for plotting two-dimmensional graph already included "gibbs.s")

# Exmple 9.1: A Markov Chain for Card-Shuffling (p.348-349)
P <- matrix(c(1/3,0,1/3,0,1/3,0,0,1/3,1/3,0,1/3,0,1/3,0,1/3,0,0,1/3,1/3,0,0,1/3,0,1/3,0,1/3,0,1/3,1/3,0,0,1/3,0,1/3,0,1/3),nrow=6)

MC.multiply <- function(P.in,N) {
  P1 <-c(1,0,0,0,0,0)%*%P.in
  for (i in 1:(N-1)) {
    P1 <- P1%*%P.in
    print(P1)
  }
  P1
}

MC.multiply(P,15)


# Example 9.2:Changepoint Analysis for Coal Mining Disasters (p.360-364) 
coal.mining.disasters <- c(4,5,4,1,0,4,3,4,0,6,3,3,4,0,2,6,3,3,5,
                           4,5,3,1,4,4,1,5,5,3,4,2,5,2,2,3,4,2,1,
                           3,2,2,1,1,1,1,3,0,0,1,0,1,1,0,0,3,1,0,
                           3,2,2,0,1,1,1,0,1,0,1,0,0,0,2,1,0,0,0,
                           1,1,0,2,3,3,1,1,2,1,1,1,1,2,4,2,0,0,0,
                           1,4,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1)

# the Gibbs sampler (seed added)
bcp <- function (theta.matrix,y,reps,a,b,g,d) {
  n <- length(y)
  k.prob <-rep(0,length=n)
  set.seed(11060018)
  for (i in 2:(reps+1)) {
    lambda <- rgamma(1, a+sum(y[1:theta.matrix[(i-1),3]]),b+theta.matrix[(i-1),3])
    phi    <- rgamma(1, g+sum(y[theta.matrix[(i-1),3]:n]),d+length(y)-theta.matrix[(i-1),3])
    for (j in 1:n) k.prob[j] <- exp(j*(phi-lambda))*(lambda/phi)^sum(y[1:j])
    k.prob <- k.prob/sum(k.prob)
    k      <- sample(1:n,size=1, prob=k.prob)
    theta.matrix <- rbind(theta.matrix,c(lambda,phi,k))
  }
  return(theta.matrix)
}

# running the Gibbs sampler 
start <- matrix(c(1,1,100),1,3)
coal.out <- bcp(start,coal.mining.disasters,2000,4,1,1,2)
summary(coal.out)

# plotting the Gibbs Sampler (scripts are from "gibbs.s" written by Jeff Gill)
plot.walk.multi <- function(walk.mat,sim.rm)  {
  walk.mat <- walk.mat[,-sim.rm]
  points(walk.mat[1,1],walk.mat[1,2])
  for(i in 1:(nrow(walk.mat)-1))  {
    segments(walk.mat[i,1],walk.mat[i,2],walk.mat[(i+1),1],walk.mat[i,2])
    segments(walk.mat[(i+1),1],walk.mat[i,2],walk.mat[(i+1),1],walk.mat[(i+1),2])
    
  }
}

start <- matrix(c(1,1,100),1,3)
coal.out <- bcp(start,coal.mining.disasters,100,4,1,1,2)
par(mfrow=c(1,2),mar=c(3,2,2,2),oma=c(3,3,1,1))

plot(coal.out,xlim=range(coal.out[,1]),ylim=range(coal.out[,3]),type="n",xlab="",ylab="")
mtext(outer=F,side=1,cex=1.1,"Lambda",line=2)
mtext(outer=F,side=2,"k",line=2.25)
plot.walk.multi(coal.out,2)

plot(coal.out,xlim=range(coal.out[,2]),ylim=range(coal.out[,3]),type="n",xlab="",ylab="")
mtext(outer=F,side=1,cex=1.1,"Phi",line=2.25)
mtext(outer=F,side=2,cex=1.1,"k",line=2)
plot.walk.multi(coal.out,1)

# Example 9.3: Changepoint Analysys for Military Fatalities in Afghanistan
nato.fatalities <- c(3,5,4,10,12,14,9,1,3,0,3,1,6,
                     1,8,4,7,12,2,2,7,2,4,2,6,8,
                     1,11,2,3,3,9,5,2,3,4,8,7,1,
                     3,2,6,19,4,29,2,33,12,10,7,4,0)

# starts from 45 
start <- matrix(c(1,1,45),1,3)
nato.out <- bcp(start,nato.fatalities,2000,1,2,4,4)
summary(nato.out)

